PROGRAM MonitorSenseLines;

USES
	SortAndSearchU, ThinkWindowsU, OffscreenU, ResourcesU;

{ CircuitDiagram }
{ 9 bits: 012 21 20 10 }
{ 012: sense 0, sense 1, sense 2 relative to monitor ground: 0 = same (ground), 1 = different (high) }
{ 21 20 10: sense 2 - 1, sense 2 - 0, sense 1 - 0: 0 = no wire, 1 = ->|- diode, 2 = -|<- diode, 3 = straight wire }

{CircuitOutput }
{ 9 bits: 012 10 20 21 }
{ 012: sense 0, sense 1, sense 2 relative to ground: 0 = same (ground), 1 = different (high) }
{ 10: sense 1 and sense 0 relative to sense 2 when sense 2 is pulled low: 0 = same (ground), 1 = different (high) }
{ 20: sense 2 and sense 0 relative to sense 1  }
{ 21: sense 2 and sense 1 relative to sense 0  }

TYPE
	SenseCircuitTable = ARRAY[0..1, 0..1, 0..1, 0..3, 0..3, 0..3] OF Integer;
	SenseCircuitTablePtr = ^SenseCircuitTable;
	SenseCircuitTable2 = ARRAY[0..511] OF Integer;
	SenseCircuitTable2Ptr = ^SenseCircuitTable2;
VAR
	senseCircuits: SenseCircuitTable2; 				{ ARRAY [CircuitDiagram] OF CircuitOutput }
	senseCircuitsByOutput: SenseCircuitTable2;	{ ARRAY[0..511] OF CircuitDiagram }
	sensePivotCircuit: Integer;

FUNCTION Reverse (s10: Integer): Integer;
{ calculates s01 from s10 }
	TYPE
		SmallArray = ARRAY[0..3] OF SignedByte;
	CONST
		reverseNumbers = $00020103;
	BEGIN
		Reverse := SmallArray(reverseNumbers)[s10];
	END;

FUNCTION CalcValue (s0, s1, s2, s21, s20, s10: Integer): Integer;
	BEGIN
		CalcValue := 1;
		IF (s0 = 0) | ((s1 = 0) & ((s10 > 1) | ((Reverse(s21) > 1) & (s20 > 1)))) | ((s2 = 0) & ((s20 > 1) | ((s21 > 1) & (s10 > 1)))) THEN
			CalcValue := 0
	END;

FUNCTION CountDiodes (circuit: Integer): Integer;
	BEGIN
		CountDiodes := ORD(BTST(circuit, 5) <> BTST(circuit, 4)) + ORD(BTST(circuit, 3) <> BTST(circuit, 2)) + ORD(BTST(circuit, 1) <> BTST(circuit, 0));
	END;

FUNCTION CountStraightWires (circuit: Integer): Integer;
	BEGIN
		CountStraightWires := ORD(BAND(BSR(circuit, 4), 3) = 3) + ORD(BAND(BSR(circuit, 2), 3) = 3) + ORD(BAND(circuit, 3) = 3);
	END;

FUNCTION CountGrounds (circuit: Integer): Integer;
	BEGIN
		CountGrounds := 3 - ORD(BTST(circuit, 8)) - ORD(BTST(circuit, 7)) - ORD(BTST(circuit, 6));
	END;

FUNCTION CompareSenseOutput (ndx: LongInt): LongInt;
	VAR
		result: LongInt;
		circuitToCompare: Integer;
	BEGIN
		circuitToCompare := senseCircuitsByOutput[ndx];
		result := 0;
		IF circuitToCompare <> sensePivotCircuit THEN
			BEGIN
				result := senseCircuits[circuitToCompare] - senseCircuits[sensePivotCircuit]; { compare outputs of circuits }
				IF result = 0 THEN { compare circuits }
					BEGIN
						result := CountDiodes(circuitToCompare) - CountDiodes(sensePivotCircuit);
						IF result = 0 THEN
							BEGIN
								result := CountStraightWires(circuitToCompare) + CountGrounds(circuitToCompare) - CountStraightWires(sensePivotCircuit) - CountGrounds(sensePivotCircuit);
								IF result = 0 THEN
									BEGIN
										result := CountStraightWires(circuitToCompare) - CountStraightWires(sensePivotCircuit);
										IF result = 0 THEN
											result := circuitToCompare - sensePivotCircuit;
									END;
							END;
					END;
			END;
		CompareSenseOutput := result;
	END;

PROCEDURE SetSensePivot (ndx: LongInt);
	BEGIN
		sensePivotCircuit := senseCircuitsByOutput[ndx];
	END;

PROCEDURE ExchangeSense (ndx1, ndx2: LongInt);
	VAR
		temp: Integer;
		int1: IntegerPtr;
		int2: IntegerPtr;
	BEGIN
		int1 := @senseCircuitsByOutput[ndx1];
		int2 := @senseCircuitsByOutput[ndx2];
		temp := int1^;
		int1^ := int2^;
		int2^ := temp;
	END;

PROCEDURE MakeCircuitOutputString (circuitOutput: Integer;
								VAR outputString: Str255);

	PROCEDURE SetOutputStringBit (bit, ndx: Integer);
		BEGIN
			IF BTST(circuitOutput, bit) THEN
				outputString[ndx] := SUCC(outputString[ndx]);
		END;

	BEGIN
		outputString := '0  0  0    00  00  00';
		SetOutputStringBit(8, 1);
		SetOutputStringBit(7, 4);
		SetOutputStringBit(6, 7);
		SetOutputStringBit(5, 12);
		SetOutputStringBit(4, 13);
		SetOutputStringBit(3, 16);
		SetOutputStringBit(2, 17);
		SetOutputStringBit(1, 20);
		SetOutputStringBit(0, 21);
	END;

FUNCTION StretchCircuitOutput (circuitOutput: Integer): LongInt;
	BEGIN
		StretchCircuitOutput := BAND(BSL(circuitOutput, 18), $0F000000) + BSL(ORD(BTST(circuitOutput, 5)), 20) + BSL(ORD(BTST(circuitOutput, 4)), 16) + BSL(ORD(BTST(circuitOutput, 3)), 12) + BSL(ORD(BTST(circuitOutput, 2)), 8) + BSL(ORD(BTST(circuitOutput, 1)), 4) + BAND(circuitOutput, 1);
	END;

PROCEDURE DrawCircuits;
	CONST
		kSpaceBetween = 6;
		kWidth = 32;
		kHeight = 22;
		kTotalWidth = kWidth + kSpaceBetween;

		kYOffsetInc = 28;
		kYOffsetSameOutputInc = 25;

		kLeftMargin = 4;
		kRightMargin = 4 + 16;
		kTopMargin = 5;
		kBottomMargin = 2 + 16;

		kTextLineHeight = 10;
		kFirstRowCircuits = 4;
		kMaxCircuits = 4; { 4 suggested, 13 for whole width,  304 for all }
	VAR
		tableNdx: Integer;
		circuitOutput: Integer;

		curMax: Integer;
		count: Integer;
		curCircuitOutput: Integer;
		circuit: Integer;
		curOffset: Integer;
		curYOffset: Integer;

		labelRect: Rect;
		s0Rect: Rect;
		s1Rect: Rect;
		s2Rect: Rect;
		s21Rect: Rect;
		s20Rect: Rect;
		s10Rect: Rect;
		tRect: Rect;

		icons: ARRAY[0..11] OF CIconHandle;

		isNewLine: Boolean;
		outputString: Str255;
		outputString2: Str255;

		outputCount: Integer;
		sameOutputCount: Integer;

		circuitCount: Integer;

		drawPort: GrafPtr;


	PROCEDURE DrawCircuitIcon (ndx, ndxOffset: Integer;
									r: Rect;
									draw0: Boolean);
		BEGIN
			IF (ndxOffset > 0) | draw0 THEN
				BEGIN
					OffsetRect(r, curOffset, curYOffset);
					PlotCIcon(r, icons[ndx + ndxOffset]);
				END;
		END;

	PROCEDURE NewPage;
		BEGIN
			IF curYOffset < 6000 THEN
				BEGIN
					Writeln('Press mouse button to continue');
					REPEAT
					UNTIL Button;
					REPEAT
					UNTIL NOT Button;
				END;
			curYOffset := kTopMargin;
			EraseRect(drawPort^.portRect);
		END;

	BEGIN { DrawCircuits }
		GetPort(drawPort);
		TextFont(128);
		TextSize(5);
		PenNormal;
		curMax := 1;
		count := 1;
		curCircuitOutput := -1;
		FOR tableNdx := 0 TO 511 DO
			BEGIN
				circuitOutput := senseCircuits[senseCircuitsByOutput[tableNdx]];
				IF circuitOutput = curCircuitOutput THEN
					count := count + 1
				ELSE
					BEGIN
						IF count > curMax THEN
							curMax := count;
						count := 1;
						curCircuitOutput := circuitOutput;
					END;
			END;

		FOR tableNdx := 0 TO 11 DO
			icons[tableNdx] := GetCIcon(128 + tableNdx);

		SetRect(labelRect, 0, 0, kWidth, kHeight);
		SetRect(s0Rect, 23, 17, 28, 22);
		SetRect(s1Rect, 14, 0, 19, 5);
		SetRect(s2Rect, 3, 17, 8, 22);
		SetRect(s21Rect, 7, 7, 15, 15);
		SetRect(s20Rect, 7, 13, 25, 20);
		SetRect(s10Rect, 17, 7, 25, 15);

		outputCount := 0;
		curYOffset := 6000;
		curCircuitOutput := -1;
		FOR tableNdx := 0 TO 511 DO
			BEGIN
				circuit := senseCircuitsByOutput[tableNdx];
				circuitOutput := senseCircuits[circuit];
				IF circuitOutput = curCircuitOutput THEN
					BEGIN
						IF circuitCount <= kMaxCircuits THEN
							curOffset := curOffset + kTotalWidth;
						IF (curOffset > drawPort^.portRect.right - kWidth - kRightMargin) | ((circuitCount = kFirstRowCircuits + 1) & (circuitCount <= kMaxCircuits)) THEN
							BEGIN
								curYOffset := curYOffset + kYOffsetSameOutputInc;
								curOffset := kLeftMargin;

								IF curYOffset > drawPort^.portRect.bottom - kHeight - kBottomMargin THEN
									NewPage;
							END;
					END
				ELSE
					BEGIN
						circuitCount := 1;
						outputCount := outputCount + 1;
						sameOutputCount := tableNdx + 1;
						WHILE (sameOutputCount < 512) & (circuitOutput = senseCircuits[senseCircuitsByOutput[sameOutputCount]]) DO
							sameOutputCount := sameOutputCount + 1;
						sameOutputCount := sameOutputCount - tableNdx;

						isNewLine := TRUE;

						IF curCircuitOutput <> -1 THEN
							; {AddResource( curCircuitoutput name and ID) }

						IF BSR(circuitOutput, 6) <> BSR(curCircuitOutput, 6) THEN
							PenSize(1, 2)
						ELSE
							PenPat(gray);

						MoveTo(0, curYOffset + kHeight + 2);
						Line(1000, 0);

						curYOffset := curYOffset + kYOffsetInc;
						curOffset := kLeftMargin;
						curCircuitOutput := circuitOutput;

						IF curYOffset > drawPort^.portRect.bottom - kHeight - kTextLineHeight - kBottomMargin THEN
							BEGIN
								NewPage;
								MoveTo(0, kTopMargin - 4);
								Line(1000, 0);
							END;
						PenNormal;
					END;

				IF isNewLine THEN
					BEGIN
						Write('.');
						isNewLine := FALSE;
						curYOffset := curYOffset + kTextLineHeight;
						MoveTo(kLeftMargin, curYOffset - 4);
						MakeCircuitOutputString(circuitOutput, outputString);
						CASE StretchCircuitOutput(circuitOutput) OF
							$0000000: 
								outputString2 := Concat('RGB 21", 21" Color, Apple 21S Color', chr(13), '1152x870 100MHz 68.7kHz 75Hz');
							$1001010: 
								outputString2 := Concat('NTSC underscan 512x384, overscan', chr(13), '640x480 12.2727MHz 15.7kHz 59.94Hz');
							$2100001: 
								outputString2 := Concat('12" Apple RGB 512x384', chr(13), '15.6672MHz 24.48kHz 60.15Hz');
							$3101011: 
								outputString2 := Concat('B&W 12", 12" Apple Monochrome, 13" Apple RGB, Hi-Res 12-14"', chr(13), '640x480 30.24MHz 35.0kHz 66.7Hz');
							$4010100: 
								outputString2 := Concat('B&W 15", Apple Portrait', chr(13), '640x870 57.2832,4MHz 68.9kHz 75Hz');
							$5011110: 
								outputString2 := Concat('RGB 15", 15" Tilt', chr(13), '57.2834MHz');
							$6110101: 
								outputString2 := Concat('B&W 21", Apple 2 Page Mono', chr(13), '1152x870 100MHz 68.7kHz 75Hz');
{ Extended Sense Line Protocol (Type 7) }
							$7000000: 
								outputString2 := Concat('PAL, NTSC/PAL (Option 1)', chr(13), 'underscan 640x480, overscan 768x576 14.75MHz 15.625kHz 50Hz');
							$7010100: 
								outputString2 := Concat('NTSC w/convolution (Alternate)', chr(13), '12.2727MHz');
							$7010111: 
								outputString2 := Concat('VGA', chr(13), '640x480 25.175MHz 31.47kHz 59.95Hz', chr(13), 'SVGA (Super VGA)', chr(13), '800x600 36MHz 35.16kHz 56Hz, 1024x768 35.16MHz 60Hz');
							$7101101: 
								outputString2 := Concat('RGB 16", 16" Color', chr(13), '832x624 57.2832,4MHz 49.7kHz 75Hz');
							$7110000: 
								outputString2 := Concat('PAL w/convolution (Alternate) (Option 2)', chr(13), 'underscan 640x480, overscan 768x576 14.75MHz 15.625kHz 50Hz');
							$7111010: 
								outputString2 := Concat('RGB 19", 19" Color', chr(13), '1024x768 80MHz');
							$7111111: 
								outputString2 := 'no-connect';
{ Extended Sense Line Protocol (Type 6) }
							$3000011: 
								outputString2 := Concat('Multiple Scan 13, 14"', chr(13), '640x480 67MHz, 832x624 75MHz');
							$3001011: 
								outputString2 := Concat('Multiple Scan 16, 17"', chr(13), '640x480 67MHz, 832x624 75MHz, 1024x768 75MHz');
							$3100011: 
								outputString2 := Concat('Multiple Scan 20, 21"', chr(13), '640x480 67MHz, 832x624 75MHz, 1024x768 75MHz, 1152x870 75MHz');
							OTHERWISE
								outputString2 := '';
						END; { CASE }
						DrawString(StringOf(outputCount : 2, ')   [ ', outputString, ' ] ', Copy('  ', 1, ORD(sameOutputCount < 10) + ORD(sameOutputCount < 100)), '(x', sameOutputCount : 0, ')  ')); { option-space is same length as digit }
						tRect.botRight := drawPort^.portRect.botRight;
						GetPen(tRect.topLeft);
						tRect.top := tRect.top - 6;
						tRect.right := tRect.right - kRightMargin;
						TextBox(@outputString2[1], Length(outputString2), tRect, teJustLeft);
					END;

				IF circuitCount <= kMaxCircuits THEN
					BEGIN
						DrawCircuitIcon(11, 0, labelRect, TRUE);
						DrawCircuitIcon(8, 1 - ORD(BTST(circuit, 8)), s0Rect, FALSE);
						DrawCircuitIcon(9, 1 - ORD(BTST(circuit, 7)), s1Rect, FALSE);
						DrawCircuitIcon(8, 1 - ORD(BTST(circuit, 6)), s2Rect, FALSE);

						DrawCircuitIcon(2, BAND(BSR(circuit, 4), 3), s21Rect, FALSE);
						DrawCircuitIcon(-1, BAND(BSR(circuit, 2), 3), s20Rect, FALSE);
						DrawCircuitIcon(5, BAND(circuit, 3), s10Rect, FALSE);

					END;
				circuitCount := circuitCount + 1;

			END; { FOR }

		MoveTo(0, curYOffset + kHeight + 2);
		PenSize(1, 2);
		Line(1000, 0);
		PenNormal;

		curYOffset := curYOffset + kYOffsetInc;
		IF curYOffset > drawPort^.portRect.bottom - kTextLineHeight * 3 - kBottomMargin THEN
			NewPage;

		curYOffset := curYOffset + kTextLineHeight;
		MoveTo(kLeftMargin, curYOffset - 4);
		DrawString(StringOf('512 different circuits producing ', outputCount : 0, ' different outputs.'));

		curYOffset := curYOffset + kTextLineHeight;
		MoveTo(kLeftMargin, curYOffset - 4);
		DrawString('Description Line: [ Sense 0  Sense 1  Sense 2  1-0(2 low)  2-0(1 low)  2-1(0 low) ]  (x No. of circuits)');

		curYOffset := curYOffset + kTextLineHeight - 2;
		MoveTo(kLeftMargin, curYOffset - 4);
		DrawString('                  Monitor Signal   Dot Frequency (MHz) Horizontal Refresh (kHz) Vertical Refresh (Hz)');
	END;

PROCEDURE MakeMonitorSenseLines;
	VAR
		s0, s1, s2, s21, s20, s10: Integer;
		tableNdx: Integer;
		circuitOutput: Integer;

	BEGIN { MakeMonitorSenseLines }

		Writeln('Calculating Circuit Diagrams');

		tableNdx := 0;
		FOR s0 := 0 TO 1 DO
			FOR s1 := 0 TO 1 DO
				FOR s2 := 0 TO 1 DO
					FOR s21 := 0 TO 3 DO
						FOR s20 := 0 TO 3 DO
							FOR s10 := 0 TO 3 DO
								BEGIN
									circuitOutput := 0;
									circuitOutput := circuitOutput + 256 * CalcValue(s0, s1, s2, s21, s20, s10);								{ sense 0 }
									circuitOutput := circuitOutput + 128 * CalcValue(s1, s0, s2, s20, s21, Reverse(s10));				{ sense 1 }
									circuitOutput := circuitOutput + 64 * CalcValue(s2, s0, s1, s10, Reverse(s21), Reverse(s20));	{ sense 2 }

									circuitOutput := circuitOutput + 32 * CalcValue(s1, s0, 0, s20, s21, Reverse(s10));				{ sense 1 when sense 2 low }
									circuitOutput := circuitOutput + 16 * CalcValue(s0, s1, 0, s21, s20, s10);								{ sense 0 when sense 2 low }

									circuitOutput := circuitOutput + 8 * CalcValue(s2, s0, 0, s10, Reverse(s21), Reverse(s20));	{ sense 2 when sense 1 low }
									circuitOutput := circuitOutput + 4 * CalcValue(s0, 0, s2, s21, s20, s10);									{ sense 0 when sense 1 low }

									circuitOutput := circuitOutput + 2 * CalcValue(s2, 0, s1, s10, Reverse(s21), Reverse(s20));	{ sense 2 when sense 0 low }
									circuitOutput := circuitOutput + CalcValue(s1, 0, s2, s20, s21, Reverse(s10));						{ sense 1 when sense 0 low }

									senseCircuits[tableNdx] := circuitOutput;
									tableNdx := tableNdx + 1;
								END; { FOR }
		FOR tableNdx := 0 TO 511 DO
			senseCircuitsByOutput[tableNdx] := tableNdx;

		Writeln('Sorting Circuit Diagrams');
		QuickSort(0, 511, CompareSenseOutput, SetSensePivot, ExchangeSense);

	END; { MakeMonitorSenseLines }

PROCEDURE InitStuff;
	VAR
		r: Rect;
	BEGIN
		SetRect(r, 0, 320, 512, 384);
		SetTextRect(r);

		SetRect(r, 0, 38, 512, 300);
		SetDrawingRect(r);

		ShowText;
		ShowDrawing;

		InitThinkWindows;
	END;


PROCEDURE DoDrawCircuits;
	VAR
		offscreenPort: GrafPtr;
		boundsRect: Rect;
		curPort: GrafPtr;
		destRect: Rect;
		thePic: PicHandle;
		err: OSErr;
	BEGIN
		Writeln('Making Offscreen Port');
		offscreenPort := NIL;
		SetRect(boundsRect, 0, 0, 512, 1800);
		IF New1BitOffscreen(offscreenPort, boundsRect) = noErr THEN
			BEGIN
				Writeln('Drawing Circuits');
				DrawCircuits;

				SetRect(destRect, 0, 0, 75, 262);
				CopyBits(offscreenPort^.portBits, ThinkDrawingWindow^.portBits, boundsRect, destRect, srcCopy, NIL);

				thePic := OpenPicture(boundsRect);
				CopyBits(offscreenPort^.portBits, offscreenPort^.portBits, boundsRect, boundsRect, srcCopy, NIL);
				ClosePicture;

				err := AddRes(thePic, 'PICT', 128, 0, 'Monitor Sense Lines Table');
				IF err <> noErr THEN
					Writeln('err ', err : 0, ' while trying to add the resource.');

				Dispose1BitOffscreen(offscreenPort, TRUE);
				Writeln;
				Writeln('Done');
			END;
	END;

BEGIN
	InitStuff;

	MakeMonitorSenseLines;

	DrawCircuits;
	DoDrawCircuits;
END.